/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// net_wins.c


#include "quakedef.h"
#include "winquake.h"
#include "net_dgrm.h"


#define MAXHOSTNAMELEN		256

#define INADDR_ANY 0
#define INADDR_BROADCAST 0xffffffff

static int net_acceptsocket = -1;		// socket for fielding new connections
static int net_controlsocket;
static int net_broadcastsocket = 0;
static struct qsockaddr broadcastaddr;

static unsigned long myAddr;


#include "net_wins.h"

UInt32 bNetLibOpened;
UInt32 gblIPAddress;
UInt32 netLibRefNum;
UInt32 gblQuakePort;

// For some odd reason...
#define ntohl(x) \
        ((unsigned long int)((((unsigned long int)(x) & 0x000000ffU) << 24) | \
                             (((unsigned long int)(x) & 0x0000ff00U) <<  8) | \
                             (((unsigned long int)(x) & 0x00ff0000U) >>  8) | \
                             (((unsigned long int)(x) & 0xff000000U) >> 24)))

#define ntohs(x) \
        ((unsigned short int)((((unsigned short int)(x) & 0x00ff) << 8) | \
                              (((unsigned short int)(x) & 0xff00) >> 8))) \
	  
#define htonl(x) ntohl(x)
#define htons(x) ntohs(x)


extern char	lanConfig_joinname[22];


int winsock_initialized = 0;
//WSADATA		winsockdata;

//unsigned long inet_addr(char *addr){
//	return 0;
//}

//=============================================================================

void WINS_GetLocalAddress()
{
	char               buff[MAXHOSTNAMELEN];
	struct sockaddr_in local;
	unsigned long      addr;
	UInt16             size = sizeof(buff);
	UInt32             crID;
	UInt16             instance;
	UInt8              isUp = false;
	Err err;
	int idx = 0;

	if (myAddr != INADDR_ANY)
		return;

	stub_NetLibSettingGet(netLibRefNum, netSettingHostName, buff, &size);
	buff[size] = 0;

	for(;;)
	{
		err = stub_NetLibIFGet(netLibRefNum, idx, &crID, &instance);

		if (err != errNone)
			break;

		//Check to see if the interface is up
		size = sizeof(isUp);
		stub_NetLibIFSettingGet(netLibRefNum, crID, instance, netIFSettingUp, &isUp, &size);
		if (isUp)
		{
			size = sizeof(myAddr);
			err = stub_NetLibIFSettingGet(netLibRefNum, crID, instance, netIFSettingActualIPAddr, &myAddr, &size);
		
			if (myAddr != 0x0100007f)
				break;
		}
		
		idx++;
	}

	addr = ntohl(myAddr);
	sprintf(my_tcpip_address, "%d.%d.%d.%d", (addr >> 24) & 0xff, (addr >> 16) & 0xff, (addr >> 8) & 0xff, addr & 0xff);
}


int WINS_Init (void)
{
	if (bNetLibOpened)
	{
		char               buff[MAXHOSTNAMELEN];
		struct sockaddr_in local;
		unsigned long      addr;
		
		myAddr = INADDR_ANY;
		strcpy(my_tcpip_address, "INADDR_ANY");

		WINS_GetLocalAddress();

		addr = gblIPAddress;
		sprintf(lanConfig_joinname, "%d.%d.%d.%d", (addr >> 24) & 0xff, (addr >> 16) & 0xff, (addr >> 8) & 0xff, addr & 0xff);

		local.sin_addr.s_addr = myAddr;
		WINS_GetNameFromAddr(&local, buff);
		Cvar_Set ("hostname", buff);

		if ((net_controlsocket = WINS_OpenSocket (0)) == -1)
		{
			menu_info("WINS_Init: Unable to open control socket\n");
			return -1;
		}

		((struct sockaddr_in *)&broadcastaddr)->sin_family = AF_INET;
		((struct sockaddr_in *)&broadcastaddr)->sin_addr.s_addr = INADDR_BROADCAST;
		((struct sockaddr_in *)&broadcastaddr)->sin_port = htons((unsigned short)net_hostport);
		
		tcpipAvailable = true;

		return net_controlsocket;
	}
	else
		return -1;

}

//=============================================================================

void WINS_Shutdown (void)
{
	WINS_Listen (false);
	WINS_CloseSocket (net_controlsocket);
}

//=============================================================================

void WINS_Listen (qboolean state)
{
	if (net_acceptsocket != -1)
		return;

	if (state)
	{
		WINS_GetLocalAddress();
		if ((net_acceptsocket = WINS_OpenSocket (net_hostport)) == -1)
			Sys_Error ("WINS_Listen: Unable to open accept socket\n");
	}
	else
	{
		WINS_CloseSocket (net_acceptsocket);
		net_acceptsocket = -1;
	}
}

//=============================================================================

int WINS_OpenSocket (int port)
{
	int newsocket;
	struct sockaddr_in address;
	unsigned long _true = 1;
	Err err;

	//if ((newsocket = psocket (PF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1)
	newsocket = (int) stub_NetLibSocketOpen (netLibRefNum, netSocketAddrINET, netSocketTypeDatagram, netSocketProtoIPUDP, 0, &err);
	if (newsocket == -1)
	{
		Sys_Error ("Unable to open socket.");
		return -1;
	}

	address.sin_family = AF_INET;
	address.sin_addr.s_addr = myAddr;
	address.sin_port = htons((unsigned short)port);
	if( stub_NetLibSocketBind (netLibRefNum, newsocket, (void *)&address, sizeof(address), 0, &err) == 0)
		return newsocket;

	Sys_Error ("Unable to bind to %s", WINS_AddrToString((struct qsockaddr *)&address));

ErrorReturn:
	stub_NetLibSocketClose (netLibRefNum, newsocket, 0, &err);

	return -1;
}

//=============================================================================

int WINS_CloseSocket (int socket)
{
	Err err;

	if (socket == net_broadcastsocket)
		net_broadcastsocket = 0;

	return stub_NetLibSocketClose (netLibRefNum, socket, 0, &err);
}


//=============================================================================
/*
============
PartialIPAddress

this lets you type only as much of the net address as required, using
the local network components to fill in the rest
============
*/
static int PartialIPAddress (char *in, struct qsockaddr *hostaddr)
{
	char buff[256];
	char *b;
	int addr;
	int num;
	int mask;
	int run;
	int port;

	buff[0] = '.';
	b = buff;
	strcpy(buff+1, in);
	if (buff[1] == '.')
		b++;

	addr = 0;
	mask=-1;
	while (*b == '.')
	{
		b++;
		num = 0;
		run = 0;
		while (!( *b < '0' || *b > '9'))
		{
		  num = num*10 + *b++ - '0';
		  if (++run > 3)
		  	return -1;
		}
		if ((*b < '0' || *b > '9') && *b != '.' && *b != ':' && *b != 0)
			return -1;
		if (num < 0 || num > 255)
			return -1;
		mask<<=8;
		addr = (addr<<8) + num;
	}

	if (*b++ == ':')
		port = Q_atoi(b);
	else
		port = net_hostport;

	hostaddr->sa_family = AF_INET;
	((struct sockaddr_in *)hostaddr)->sin_port = htons((short)port);
	((struct sockaddr_in *)hostaddr)->sin_addr.s_addr = (myAddr & htonl(mask)) | htonl(addr);

	return 0;
}
//=============================================================================

int WINS_Connect (int socket, struct qsockaddr *addr)
{
	return 0;
}

//=============================================================================

int WINS_CheckNewConnections (void)
{
	Err err;
	char buf[4096];

	if (net_acceptsocket == -1)
		return -1;

	if (bNetLibOpened)
	{
		UInt16 fromLength = sizeof(struct qsockaddr);
		struct qsockaddr fromAddr;

		if (stub_NetLibReceive(netLibRefNum, net_acceptsocket, buf, sizeof(buf), netIOFlagPeek, (NetSocketAddrType *) &fromAddr, &fromLength, 0, &err) > 0)
			return net_acceptsocket;
	}
	
	return -1;
}

//=============================================================================

int WINS_Read (int socket, byte *buf, int len, struct qsockaddr *addr)
{
	int c = 0;
	Err err;

	if (bNetLibOpened)
	{
		UInt16 fromLength = sizeof(struct qsockaddr);
		c = stub_NetLibReceive(netLibRefNum, socket, buf, len, 0, (NetSocketAddrType *) addr, &fromLength, 0, &err);
		if (c == -1 )
		{
			if (err != netErrTimeout)
				Sys_PrintF ("ReadPacket error: 0x%x", err);
			c = 0;
		}
	}
	
	return c;
}

//=============================================================================

int WINS_MakeSocketBroadcastCapable (int socket)
{
	net_broadcastsocket = socket;

	return 0;
}

//=============================================================================

int WINS_Broadcast (int socket, byte *buf, int len)
{
	return len;
}

//=============================================================================

int WINS_Write (int socket, byte *buf, int len, struct qsockaddr *addr)
{
	int c = 0;
	Err err;

	if (bNetLibOpened)
	{
		c = stub_NetLibSend(netLibRefNum, socket, buf, len, 0, addr, sizeof(struct qsockaddr), 0, &err);
		if (c == -1 )
		{
			Sys_PrintF ("SendPacket error: 0x%x", err);
			c = 0;
		}
	}
	
	return c;
}

//=============================================================================

char *WINS_AddrToString (struct qsockaddr *addr)
{
	static char buffer[22];

	int haddr;

	haddr = ntohl(((struct sockaddr_in *)addr)->sin_addr.S_un.S_addr);
	sprintf(buffer, "%d.%d.%d.%d:%d", (haddr >> 24) & 0xff, (haddr >> 16) & 0xff, (haddr >> 8) & 0xff, haddr & 0xff, ntohs(((struct sockaddr_in *)addr)->sin_port));

	return buffer;
}

//=============================================================================

int WINS_StringToAddr (char *string, struct qsockaddr *addr)
{
	int ha1, ha2, ha3, ha4, hp;
	int ipaddr;

	sscanf(string, "%d.%d.%d.%d:%d", &ha1, &ha2, &ha3, &ha4, &hp);
	ipaddr = (ha1 << 24) | (ha2 << 16) | (ha3 << 8) | ha4;

	addr->sa_family = AF_INET;
	((struct sockaddr_in *)addr)->sin_addr.s_addr = htonl(ipaddr);
	((struct sockaddr_in *)addr)->sin_port = htons((unsigned short)hp);

	return 0;
}

//=============================================================================

int WINS_GetSocketAddr (int socket, struct qsockaddr *addr)
{
	int addrlen = sizeof(struct qsockaddr);
	unsigned int a;
	struct qsockaddr tmp_addr;
	int tmp_len = sizeof(struct qsockaddr);
	Err err;

	Q_memset(addr, 0, sizeof(struct qsockaddr));
	stub_NetLibSocketAddr(netLibRefNum, socket, (struct sockaddr *)addr, &addrlen, &tmp_addr, &tmp_len, 0, &err);
	a = ((struct sockaddr_in *)addr)->sin_addr.s_addr;
	if ((a == 0) || (a == 0x0100007f))//inet_addr("127.0.0.1"))
		((struct sockaddr_in *)addr)->sin_addr.s_addr = myAddr;

	return 0;
}

//=============================================================================

int WINS_GetNameFromAddr (struct qsockaddr *addr, char *name)
{
	NetHostInfoBufType hostbuffer;
	NetHostInfoPtr hostentry;
	Err err;

	hostentry = (NetHostInfoPtr) stub_NetLibGetHostByAddr(netLibRefNum, (char *)&((struct sockaddr_in *)addr)->sin_addr, sizeof(struct in_addr), AF_INET, &hostbuffer, 0, &err);
	if (hostentry)
	{
		Q_strncpy (name, (char *)hostentry->nameP, NET_NAMELEN - 1);
		return 0;
	}

	Q_strcpy (name, WINS_AddrToString (addr));

	return 0;
}

//=============================================================================

int WINS_GetAddrFromName(char *name, struct qsockaddr *addr)
{
	NetHostInfoBufType hostbuffer;
	NetHostInfoPtr hostentry;
	Err err;

	if (name[0] >= '0' && name[0] <= '9')
		return PartialIPAddress (name, addr);

	hostentry = (NetHostInfoPtr) stub_NetLibGetHostByName (netLibRefNum, name, &hostbuffer, 0, &err);
	if (!hostentry)
		return -1;

	addr->sa_family = AF_INET;
	((struct sockaddr_in *)addr)->sin_port = htons((unsigned short)net_hostport);
	((struct sockaddr_in *)addr)->sin_addr.s_addr = *(int *)hostentry->addrListP[0];

	return 0;
}

//=============================================================================

int WINS_AddrCompare (struct qsockaddr *addr1, struct qsockaddr *addr2)
{
	if (addr1->sa_family != addr2->sa_family)
		return -1;

	if (((struct sockaddr_in *)addr1)->sin_addr.s_addr != ((struct sockaddr_in *)addr2)->sin_addr.s_addr)
		return -1;

	if (((struct sockaddr_in *)addr1)->sin_port != ((struct sockaddr_in *)addr2)->sin_port)
		return 1;

	return 0;
}

//=============================================================================

int WINS_GetSocketPort (struct qsockaddr *addr)
{
	return ntohs(((struct sockaddr_in *)addr)->sin_port);
}


int WINS_SetSocketPort (struct qsockaddr *addr, int port)
{
	((struct sockaddr_in *)addr)->sin_port = htons((unsigned short)port);
	
	return 0;
}

//=============================================================================
