	/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// sys_win.c -- Win32 system interface code



#include "quakedef.h"
#include "winquake.h"
//#include "../resource.h"
//#include "../zlib/zlib.h"
//#include <gx.h>
//#include <Aygshell.h>
#include "conproc.h"
#include "stdfile.h"
#undef VERSION

#include <TwKeys.h>
#include <(Other SDKs)/Handspring/Include/Common/System/HsKeyCodes.h>

#include "PNOMain.h"
#include "pnobridge.h"


#define MINIMUM_WIN_MEMORY		(MINIMUM_MEMORY+1000000)
#define MAXIMUM_WIN_MEMORY		0x1000000

#define CONSOLE_ERROR_TIMEOUT	60.0	// # of seconds to wait on Sys_Error running
					//  dedicated before exiting
#define PAUSE_SLEEP		50	// sleep time on pause or minimization
#define NOT_FOCUS_SLEEP		20	// sleep time when not focus


static double ticks_per_sec_f;


volatile int		sys_checksum;
qboolean		isDedicated;
static double		lastcurtime = 0.0;
static double		curtime = 0.0;
static qboolean		sc_return_on_enter = false;
static double		pfreq;
static int		lowshift;
//HANDLE			hinput, houtput;

int SCREENWIDTH=320;
int SCREENHEIGHT=320;
int boolKeyboard=0;
int nShowScreenButtons=0;
char lastkey = 0;
char currkey = 0;
#define KEY_WIDTH 25
#define KEY_HEIGHT 25

extern UInt32 bTreo600SDK;
extern UInt32 bTreo650SDK;
extern UInt32 bDebugFile;

static char			*tracking_tag = "Clams & Mooses";

static int exit_quake=0;

#define	KEYBUF_SIZE	256
static int		keybuf[KEYBUF_SIZE];
static int		keybuf_head=0;
static int		keybuf_tail=0;

void MaskExceptions (void);
void Sys_InitFloatTime (void);
void Sys_PushFPCW_SetHigh (void);
void Sys_PopFPCW (void);

/*
================
Sys_PageIn
================
*/
void Sys_PageIn (void *ptr, int size)
{
	byte	*x;
	int		/*j, */m, n;

// touch all the memory to make sure it's there. The 16-page skip is to
// keep Win 95 from thinking we're trying to page ourselves in (we are
// doing that, of course, but there's no reason we shouldn't)
	x = (byte *)ptr;

	for (n=0 ; n<4 ; n++)
	{
		for (m=0 ; m<(size - 16 * 0x1000) ; m += 4)
		{
			sys_checksum += *(int *)&x[m];
			sys_checksum += *(int *)&x[m + 16 * 0x1000];
		}
	}
}


/*
===============================================================================

FILE IO

===============================================================================
*/

//Dan: This struct is used to support both stdio and gzio file IO.
struct SYS_HANDLE {
  FILE *f;
  //F_HANDLE f;
  int compressed;
};

#define	MAX_HANDLES		10
struct SYS_HANDLE sys_handles[MAX_HANDLES];

int		findhandle (void)
{
	int		i;

	for (i=1 ; i<MAX_HANDLES ; i++)
		if (!sys_handles[i].f)
			return i;
	Sys_Error ("out of handles");
	return -1;
}

/*
================
filelength
================
*/
//int filelength (FILE *f)
int filelength (struct SYS_HANDLE *sh)
{
	int		pos;
	int		end;

	pos = ftell (sh->f);
	fseek (sh->f, 0, SEEK_END);
	end = ftell (sh->f);
	fseek (sh->f, pos, SEEK_SET);

	return end;
}

int Sys_FileOpenRead (char *path, int *hndl)
{
//	FILE	*f;
	int		i, retval;
	int		compressed=false;
	int		size;
	//F_HANDLE	f;
	FILE *f;


	if ( path [ 0 ] == '.' && path [ 1 ] == '/' ) {
	  path += 2;
	}

	if ( path [ 0 ] == '\\' ) {
	  path += 1;
	}

	//TODO: make sure a handle was returned
	i = findhandle ();

	f = fopen(path, "rb");

	if ( ! f )
	{
		*hndl = -1;
		retval = -1;
	}
	else
	{

	  size = f -> size;

		sys_handles[i].f = f;
		sys_handles[i].compressed=0;

		*hndl = i;
		//retval = 1;//filelength(&sys_handles[i]);
		retval = f -> size;//filelength(&sys_handles[i]);
	}

	return retval;
}

int Sys_FileOpenWrite (char *path)
{
	FILE	*f;
	//F_HANDLE f;
	int		i;


	i = findhandle ();

	f = fopen(path, "wb");
	sys_handles[i].f = f;
	sys_handles[i].compressed = false;

	return i;
}

void Sys_FileClose (int handle)
{
  fclose (sys_handles[handle].f);

  sys_handles[handle].f = 0;
}

void Sys_FileSeek (int handle, int position)
{
	int old;
	fseek (sys_handles[handle].f, position, SEEK_SET);


}

int Sys_FileRead (int handle, void *dest, int count)
{
	int		x;

	x = fread (dest, 1, count, sys_handles[handle].f);
	return x;
}

int Sys_FileWrite (int handle, void *data, int count)
{
	int		x;
	x = fwrite (data, 1, count, sys_handles[handle].f);
	return count;
}

int	Sys_FileTime (char *path)
{
	FILE	*f;
	int		retval;


	f = fopen(path, "rb");
	if (f)
	{
	  fclose(f);
	  retval = 1;
	}
	else
	{
	  retval = -1;
	}

	return retval;

/*
//	f = fopen(path, "rb");
	if (f)
	{
		fclose(f);
		retval = 1;
	}
	else
	{
		retval = -1;
	}

	return retval;
*/
}

void Sys_mkdir (char *path)
{

  menu_info ( "MAKE DIRECTORY? ARRRRGH!" );
}


/*
===============================================================================

SYSTEM IO

===============================================================================
*/

/*
================
Sys_MakeCodeWriteable
================
*/
void Sys_MakeCodeWriteable (unsigned long startaddr, unsigned long length)
{
}


//#ifndef _M_IX86

void Sys_SetFPCW (void)
{
}

void Sys_PushFPCW_SetHigh (void)
{
}

void Sys_PopFPCW (void)
{
}

void MaskExceptions (void)
{
}

//#endif

/*
================
Sys_Init
================
*/
void Sys_Init (void)
{
	Sys_InitFloatTime ();
/*	MaskExceptions ();
	Sys_SetFPCW ();

	if (!QueryPerformanceFrequency (&PerformanceFreq))
		Sys_Error ("No hardware timer available");

	// get 32 out of the 64 time bits such that we have around
	// 1 microsecond resolution
	lowpart = (unsigned int)PerformanceFreq.LowPart;
	highpart = (unsigned int)PerformanceFreq.HighPart;
	lowshift = 0;

	while (highpart || (lowpart > 2000000.0))
	{
		lowshift++;
		lowpart >>= 1;
		lowpart |= (highpart & 1) << 31;
		highpart >>= 1;
	}

	pfreq = 1.0 / (double)lowpart;

	

	WinNT=false;
*/
}


void Sys_Error (char *error, ...)
{
  menu_info ( error );
//  while(1);

}

void Sys_Printf (char *fmt, ...)
{
//	menu_info(fmt);
}

void Sys_Quit (void)
{

	Host_Shutdown();
	
	exit_quake=1;
}



double temptime = 0;

/*
================
Sys_FloatTime
================
*/
double Sys_FloatTime (void)
{
	curtime = (double)TimGetTicks();
    return (curtime-lastcurtime)/ticks_per_sec_f;
}


/*
================
Sys_InitFloatTime
================
*/
void Sys_InitFloatTime (void)
{
	ticks_per_sec_f = (double)SysTicksPerSecond();
	lastcurtime = (double)TimGetTicks();
	curtime = lastcurtime;
}


char *Sys_ConsoleInput (void)
{
	return NULL;
}

void Sys_Sleep (void)
{
//	Sleep (1);
}

extern int vid_paused;





void Sys_TrapKeys()
{
	int key = 0;

	keybuf[keybuf_head] = key;
	keybuf_head = (keybuf_head + 1) & (KEYBUF_SIZE-1);
}

#define KEY_UP				0x00000001
#define KEY_DOWN			0x00000002
#define KEY_LEFT			0x00000004
#define KEY_RIGHT			0x00000008
#define KEY_RUN				0x00000010
#define KEY_JUMP			0x00000020
#define KEY_LOOK			0x00000040
#define KEY_STRAFE			0x00000080
#define KEY_STRAFE_L		0x00000100
#define KEY_STRAFE_R		0x00000200
#define KEY_FIRE			0x00000400
#define KEY_CHANGEWEAPON	0x00000800
#define KEY_STAT 			0x00001000
#define KEY_MENU			0x00002000

#define KEY_SWIMUP			0x00004000
#define KEY_SWIMDOWN		0x00008000
#define KEY_LOOKUP			0x00010000
#define KEY_LOOKDOWN		0x00020000
#define KEY_LOOKCENTER		0x00040000


unsigned int g_translateBtn[20]=
{KEY_UP,KEY_DOWN,KEY_LEFT,KEY_RIGHT,
KEY_RUN,KEY_JUMP,KEY_LOOK,
KEY_STRAFE,KEY_STRAFE_L,KEY_STRAFE_R,
KEY_FIRE,KEY_CHANGEWEAPON,KEY_STAT,KEY_MENU,
KEY_SWIMUP,KEY_SWIMDOWN,KEY_LOOKUP,KEY_LOOKDOWN,KEY_LOOKCENTER,0};

extern int g_confinput[20];
char consoleCmd[256];

void Sys_SendKeyEvents (void)
{
//  int key;
  static UInt32 old_key=0;
  static UInt32 pollcpt=0;
  UInt32 key,keyval,i;
  static UInt32 penX,penY,penDown;
  
  pollcpt++;
//	if (pollcpt&3) return;
  
#define IsPress(a) (((keyval&a)==a)&&!((old_key&a)==a))
#define IsNotPress(a) (!((keyval&a)==a)&&((old_key&a)==a))
#define HandleKey(a,b) {\
	if (IsPress(a)){		\
		Key_Event(b,true);\
	}\
	if (IsNotPress(a)){\
		Key_Event(b,false);\
	}}

	
	
	keyval=0;
	key = KeyCurrentState();		
	
	penDown=0;
	/*if (!(pollcpt&3))*/{
		EvtGetPen(&penX,&penY,&penDown);
		
	}
	//handle sys event
#ifndef ENABLE_ZGRAB	
	if (!(pollcpt&31)) {YDK_HANDLE_EVENT()}
#else
	{YDK_HANDLE_EVENT()}
#endif	
	//avoid autoff
	//every 1024 frame => 
	if (!(pollcpt&1023)) EvtResetAutoOffTimer();
	;					
	
	if (penDown){
		PointType pt;
		pt.x=penX;
		pt.y=penY;
		EvtEnqueuePenPoint(&pt);
	}
	
#define SWAP(x) (((x>>8)&0x00ff) | ((x<<8)&0xff00))
	if ((bTreo600SDK || bTreo650SDK) && bDebugFile)
	{
		UInt16 keyCodes[50] = {SWAP(keyBackspace), SWAP(keyTab), SWAP(keyReturn), SWAP(keyEscape), SWAP(keySpace), SWAP(keySingleQuote), 
		                     SWAP(keyComma), SWAP(keyDash), SWAP(keyPeriod), SWAP(keySlash), SWAP(keyZero), SWAP(keyOne), SWAP(keyTwo), SWAP(keyThree),
		                     SWAP(keyFour), SWAP(keyFive), SWAP(keySix), SWAP(keySeven), SWAP(keyEight), SWAP(keyNine), SWAP(keySemiColon), SWAP(keyLessThan),
		                     SWAP(keyEquals), SWAP(keyBackslash), SWAP(keyA), SWAP(keyB), SWAP(keyC), SWAP(keyD), SWAP(keyE), SWAP(keyF), SWAP(keyG), SWAP(keyH), SWAP(keyI),
		                     SWAP(keyJ), SWAP(keyK), SWAP(keyL), SWAP(keyM), SWAP(keyN), SWAP(keyO), SWAP(keyP), SWAP(keyQ), SWAP(keyR), SWAP(keyS), SWAP(keyT), SWAP(keyU), SWAP(keyV),
		                     SWAP(keyW), SWAP(keyX), SWAP(keyY), SWAP(keyZ)};
		Boolean keyPressed[50] = {0};

		UInt16 res = stub_HsKeysPressed(50, keyCodes, keyPressed);
		if (res > 0)
		{
			Con_DebugLog("", "res: %d\n", res);
			for (i=0; i<50; i++)
			{
				if (keyPressed[i])
				{
					Con_DebugLog("", "key pressed: %d (0x%x)\n", i, keyCodes[i]);
				}
			}
		}
	}

	penX*=_PEN_ADJUST_FACTORX_;
	penY*=_PEN_ADJUST_FACTORY_;
	if (penDown)
	{
		if (boolKeyboard)
		{
			int y = (penY - (SCREENHEIGHT/*display_height*/ - (5 * KEY_HEIGHT) - (KEY_HEIGHT / 2))) / KEY_HEIGHT;
			
			currkey = 0;
			
			if (y == 0)
			{
				int x = penX / KEY_WIDTH;

				if (x == 0) currkey = '1';
				else if (x == 1) currkey = '2';
				else if (x == 2) currkey = '3';
				else if (x == 3) currkey = '4';
				else if (x == 4) currkey = '5';
				else if (x == 5) currkey = '6';
				else if (x == 6) currkey = '7';
				else if (x == 7) currkey = '8';
				else if (x == 8) currkey = '9';
				else if (x == 9) currkey = '0';
				else if (x == 10) currkey = K_BACKSPACE;
			}
			else if (y == 1)
			{
				int x = (penX - (KEY_WIDTH / 2)) / KEY_WIDTH;

				if (x == 0) currkey = 'Q';
				else if (x == 1) currkey = 'W';
				else if (x == 2) currkey = 'E';
				else if (x == 3) currkey = 'R';
				else if (x == 4) currkey = 'T';
				else if (x == 5) currkey = 'Y';
				else if (x == 6) currkey = 'U';
				else if (x == 7) currkey = 'I';
				else if (x == 8) currkey = 'O';
				else if (x == 9) currkey = 'P';
				else if (x == 10) currkey = '~';
			}
			else if (y == 2)
			{
				int x = (penX - KEY_WIDTH) / KEY_WIDTH;

				if (x == 0) currkey = 'A';
				else if (x == 1) currkey = 'S';
				else if (x == 2) currkey = 'D';
				else if (x == 3) currkey = 'F';
				else if (x == 4) currkey = 'G';
				else if (x == 5) currkey = 'H';
				else if (x == 6) currkey = 'J';
				else if (x == 7) currkey = 'K';
				else if (x == 8) currkey = 'L';
				else if ((x >= 9) && (x <= 10)) currkey = K_ENTER;
			}
			else if (y == 3)
			{
				int x = (penX - KEY_WIDTH - (KEY_WIDTH / 2)) / KEY_WIDTH;

				if (x == 0) currkey = 'Z';
				else if (x == 1) currkey = 'X';
				else if (x == 2) currkey = 'C';
				else if (x == 3) currkey = 'V';
				else if (x == 4) currkey = 'B';
				else if (x == 5) currkey = 'N';
				else if (x == 6) currkey = 'M';
				else if (x == 7) currkey = '-';
				else if (x == 8) currkey = '.';
			}
			else if (y == 4)
			{
				int x = (penX - (2 * KEY_WIDTH)) / KEY_WIDTH;

				if ((x >= 0) && (x <= 5)) currkey = ' ';
				else if ((x == 6) || (x == 7)) boolKeyboard = 0;
			}
			if (lastkey != currkey)
			{
				if (lastkey != 0)
					Key_Event(lastkey, false);
				if (currkey != 0)
					Key_Event(currkey, true);
			}
			lastkey = currkey;
		}
		else
		{
		    nShowScreenButtons = 10;
		 	if ((penY>0)&&(penY<60)&&(penX>SCREENWIDTH-80)&&(penX<SCREENWIDTH)) key=BTN_TR;
		 	else if ((penY>(SCREENHEIGHT>>1)-30)&&(penY<(SCREENHEIGHT>>1)+30)&&(penX>SCREENWIDTH-80)&&(penX<SCREENWIDTH)) key=BTN_MR;
			else if ((penY>SCREENHEIGHT-60)&&(penY<SCREENHEIGHT)&&(penX>SCREENWIDTH-80)&&(penX<SCREENWIDTH)) key=BTN_BR;
			else if ((penY>0)&&(penY<60)&&(penX>0)&&(penX<80)) key=BTN_TL;
			else if ((penY>(SCREENHEIGHT>>1)-30)&&(penY<(SCREENHEIGHT>>1)+30)&&(penX>0)&&(penX<80)) key=BTN_ML;
			else if ((penY>SCREENHEIGHT-60)&&(penY<SCREENHEIGHT)&&(penX>0)&&(penX<80)) key=BTN_BL;
			else if ((penY>0)&&(penY<60)&&(penX>(SCREENWIDTH>>1)-40)&&(penX<(SCREENWIDTH>>1)+40)) key=BTN_TM;
		 	else if ((penY>SCREENHEIGHT-60)&&(penY<SCREENHEIGHT)&&(penX>(SCREENWIDTH>>1)-40)&&(penX<(SCREENWIDTH>>1)+40)) key=BTN_BM;
		 	else if ((penY>(SCREENHEIGHT>>1)-30)&&(penY<(SCREENHEIGHT>>1))&&(penX>(SCREENWIDTH>>1)-40)&&(penX<(SCREENWIDTH>>1)+40)) {
	 			char *c;
	 			int result;
		 		stub_consoleCmd(consoleCmd);
		 		YDK_HANDLE_EVENT_NOFUNC()
	 			if (strcmp(consoleCmd,"quit")==0){
	 				Key_Event('y',true);
		 			Key_Event('y',false);
		 		}
	 			strcat(consoleCmd,"\n");
	 			Cbuf_InsertText(consoleCmd);	 		
		 	}
		 	else if ((penY>(SCREENHEIGHT>>1))&&(penY<(SCREENHEIGHT>>1)+30)&&(penX>(SCREENWIDTH>>1)-40)&&(penX<(SCREENWIDTH>>1)+40)) {
				boolKeyboard = 1;
			}
		}
	}
	else if (lastkey != 0)
	{
		Key_Event(lastkey, false);
		lastkey = 0;
	}
	
	//if ((key==BTN_TM)||(key==BTN_BM)||(key==BTN_TL)||(key==BTN_TR)||(key==BTN_ML)||(key==BTN_MR)||(key==BTN_BL)||(key==BTN_BR)){
	if (penDown){
		if (boolKeyboard == 0)
		{
			for (int i=0;i<20;i++)
				 if (g_confinput[i]!=0)
					if (key==g_confinput[i]) {keyval=g_translateBtn[i];break;}
		}
	}
	else {
		for (i=0;i<20;i++)
			if (g_confinput[i]!=0)
				if ( (key & g_confinput[i])==g_confinput[i]) keyval|=g_translateBtn[i];	
	}
	
	HandleKey(KEY_UP,K_UPARROW)
	HandleKey(KEY_DOWN,K_DOWNARROW)
	HandleKey(KEY_LEFT,K_LEFTARROW)
	HandleKey(KEY_RIGHT,K_RIGHTARROW)
	HandleKey(KEY_RUN,K_SHIFT)
	HandleKey(KEY_JUMP,K_ENTER)
	HandleKey(KEY_LOOK,K_INS)
	HandleKey(KEY_STRAFE,K_ALT)
	HandleKey(KEY_STRAFE_L,',')
	HandleKey(KEY_STRAFE_R,'.')	
	HandleKey(KEY_FIRE,K_CTRL)
	HandleKey(KEY_CHANGEWEAPON,'/')
	HandleKey(KEY_STAT,K_TAB)
	HandleKey(KEY_MENU,K_ESCAPE)
	HandleKey(KEY_SWIMUP,'d')
	HandleKey(KEY_SWIMDOWN,'c')
	HandleKey(KEY_LOOKUP,K_PGDN)
	HandleKey(KEY_LOOKDOWN,K_DEL)
	HandleKey(KEY_LOOKCENTER,K_END)
						
	HandleKey((KEY_STAT|KEY_STRAFE_L|KEY_STRAFE_R),'~')
	

	old_key=keyval;
	
#undef IsPress
#undef IsNotPress
#undef HandleKey
}


/*
==============================================================================

 WINDOWS CRAP

==============================================================================
*/


/*
==================
WinMain
==================
*/
void SleepUntilInput (int time)
{
}


/*
==================
WinMain
==================
*/
char		*argv[MAX_NUM_ARGVS];
static char *empty_string = "";


double	win_time;



extern char pak_dir[256];

void GpMain(void *gparg)
{
	quakeparms_t	parms;
	double		oldtime, time;
	char	cwd[1024] = "";
	int				t;
	
//	StrCat ( cwd, VFSPATH );
	
	StrCat ( cwd, "ID1");

	if (cwd[Q_strlen(cwd)-1] == '/')
		cwd[Q_strlen(cwd)-1] = 0;

	parms.basedir = cwd;
	parms.cachedir = NULL;
	
	
	
	strupr(pak_dir);
	if (strcmp(pak_dir,"ID1")){
		parms.argc = 4;
		argv[0] = empty_string;
		argv[1] = "-game";
		argv[2] = pak_dir;
	}else{
		parms.argc = 0;
		argv[0] = empty_string;
	}
	argv[parms.argc+1]=empty_string;
	parms.argv = argv;

/*	parms.argc = 1;
	argv[0] = empty_string;

	argv[parms.argc+1]=empty_string;;
	parms.argv = argv;*/


//	menu_info ( cwd );

	COM_InitArgv (parms.argc, parms.argv);

	parms.argc = com_argc;
	parms.argv = com_argv;

	isDedicated = 0;
	

	

	parms.memsize = MINIMUM_WIN_MEMORY;

	while (parms.memsize >= MINIMUM_MEMORY) {		
		parms.membase = (void*)gm_malloc (parms.memsize);
		if (!parms.membase) parms.memsize-=500000;
		else break;
	}

	if (!parms.membase)
		Sys_Error ("Not enough memory free; check disk space\n");
	

	Sys_Init ();


	//Sys_Printf ("Host_Init\n");


	Host_Init (&parms);


	//GpError("Host_Init", 0);


	//while ( 1 );
	//return; //hack


	// Set up keyboard polling

	if (volume.value==0) S_BlockSound();
	
//	Cbuf_InsertText("timedemo demo1\n");



	time = 0.1;
	oldtime = Sys_FloatTime ();


    /* main window message loop */
	while (1)
	{
	
	
	    Host_Frame ((float)time);
		win_time = Sys_FloatTime ();
		time = win_time - oldtime;
		oldtime = win_time;
		
		if (exit_quake) break;
	}



	//while (parms.argc--)
	//gm_free(argv[parms.argc]);

}

//Dan:
void Sys_HighFPPrecision (void)
{
}

void Sys_LowFPPrecision (void)
{
}
